/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at51.c		main at51 program
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "at51defs.h"
#include "at51status.h"
#include "version.h"

#include "at51param.h"
#include "at51help.h"
#include "at51decision.h"

/* VARIABLES */
/* Incoming param struct */
at51_param_t at51_param;
int parm_error, parm_status;
int dec_error, dec_status;

int main(int argn, char **args) {
    char copt;
    int Mres;

    if (argn == 1) {
	at51_help_show_help_basic(&at51_param);
	return 0;
    }
    
    /* INITIALIZATIONS */
    if (at51_param_init_struct(&at51_param, &parm_error, &parm_status) == -1) {
	printf("Error: " _AT51_PROGRAM_NAME " initialization struct error: %d (%s) %d\n",
	    parm_error, strerror(parm_error), parm_status);
	exit(0);
    }
    
    /* PROCESS INCOMING PARAMS */
    while (1) {
	if ((copt = getopt(argn, args, "p:f:trwebvlh12340F")) == -1) break;
	switch (copt) {
	    case 'p': /* part name */
		if (strlen(optarg) < 30)
		     strcpy(at51_param.partname, optarg);
		break;
	    case 't': /* test command - get and check id */
		at51_param.command |= _CMD_TEST;
		break;
	    case 'r': /* read command */
		at51_param.command |= _CMD_READ;
		break;
	    case 'b': /* burn command */
		at51_param.command |= _CMD_BURN;
		break;
	    case 'w': /* write and verify command */
		at51_param.command |= _CMD_WRITE;
		break;
	    case 'e': /* erase command */
		at51_param.command |= _CMD_ERASE;
		break;
	    case '1': /* lock level 1 command */
		at51_param.command |= _CMD_LOCK1;
		break;
	    case '2': /* lock level 2 command */
		at51_param.command |= _CMD_LOCK2;
		break;
	    case '3': /* lock level 3 command */
		at51_param.command |= _CMD_LOCK3;
		break;
	    case '4': /* lock level 4 command */
		at51_param.command |= _CMD_LOCK4;
		break;
	    case '0': /* read device memory lock bits command */
		at51_param.command |= _CMD_READLOCK;
		break;
	    case 'v': /* version */
		printf(_AT51_PROGRAM_NAME" version "_AT51_PROGRAM_VERSION"\n");
		/* Free allocated resources, restore and close serial port */
		if (at51_param_finalize_struct(&at51_param, &parm_error, &parm_status) == -1) {
		    printf("Error: " _AT51_PROGRAM_NAME " finalization struct error: %d (%s) %d\n", 
			    parm_error, strerror(parm_error), parm_status);
		}
		exit(0);
	    case 'h': /* help */
		at51_param.showhelp = _Yes;
		break;
	    case 'l': /* list of supported devices */
		at51_param.showlist = _Yes;
		break;
	    case 'f': /* input / output file name */
		if (strlen(optarg) < 256)
		     strcpy(at51_param.filename, optarg);
		break;
	    case 'F': /* force doing command */
		at51_param.forcedcommand = _Yes;
		break;
	    default:
		break;
	} /* end of switch (copt) */
    } /* end of while (1) of incoming params */

    /* Lookup partname in db and get partindex */
    if (at51_dec_lookup_partindex(&at51_param) == -1)
	at51_help_show_help_device_isnt_supported(&at51_param);

    /* Check requested commands compillance */
    if (at51_dec_check_command_compillance(&at51_param) == -1)
	at51_help_show_help_compillance(&at51_param);
    
    /* Help if requested... */
    at51_help_show_help_full(&at51_param);

    /* Show intro help and continue processing device */
    at51_help_show_help_intro(&at51_param);

    /* Process device with command */
    Mres = at51_dec_process_device_command(&at51_param, &dec_error, &dec_status);
    if (Mres == -1) {
	printf("Error: processing device - %d (%s) %d\n",dec_error, strerror(dec_error), dec_status);
    }

    /* Free allocated resources, restore and close serial port */
    if (at51_param_finalize_struct(&at51_param, &parm_error, &parm_status) == -1) {
	printf("Error: " _AT51_PROGRAM_NAME " finalization struct error: %d (%s) %d\n", 
	    parm_error, strerror(parm_error), parm_status);
	exit(0);
    }
    return 0;
} /* end of main() */

/* end of at51.c */
